
;
; Load the ROM, and read it into Memory
;
LoadRom proc uses eax esi hWnd:HWND
    LOCAL ofn:OPENFILENAME
    LOCAL hFile:HANDLE
    LOCAL NumberOfBytesRead:DWORD
    LOCAL Code:WORD
    
    ;Zero struct
    invoke RtlZeroMemory,addr ofn,SIZEOF OPENFILENAME
    ; Fill struct with information
    mov ofn.lStructSize, SIZEOF OPENFILENAME
    push hWnd
    pop ofn.hwndOwner
    mov ofn.lpstrFilter, offset FileFilter
    mov ofn.lpstrFile, offset szFileName
    mov ofn.nMaxFile, MAX_PATH
    mov ofn.Flags, OFN_FILEMUSTEXIST or OFN_PATHMUSTEXIST or OFN_LONGNAMES or OFN_EXPLORER or OFN_HIDEREADONLY
    
    .if (EmulatorRunning==TRUE)
        mov PauseCpu,FALSE
    .endif
    
    ; Open File Selecting Dialog
    invoke GetOpenFileName, ADDR ofn
    
    ; If OK Was pressed
    .if (eax!=NULL)
        invoke CreateFile,addr szFileName,\
             GENERIC_READ,FILE_SHARE_READ,\
             NULL,OPEN_ALWAYS,FILE_ATTRIBUTE_NORMAL,NULL
        
        ; Check if valid file
        .if (eax==INVALID_HANDLE_VALUE)
            ret
        .endif
        
        ; Handle to the Loaded File
        mov hFile,eax
        
        ; Init Emulator (Visual Settings)
        invoke InitEmulator
        
        ;Init CPU Insights, buffers,registers,memory
        invoke Init_CPU
        
        invoke GetFileSize,hFile,NULL
        mov FileSize,eax
        
        mov esi,offset Memory
        add esi,200h
        ;Read the ROM File into Memory at location 200h - ???h (<=FFFh) 
        invoke ReadFile,hFile,esi,FileSize,addr NumberOfBytesRead,NULL
        
        ;Free the File Handle
        invoke CloseHandle,hFile
        
        ; Flag values
        mov EmulatorRunning,TRUE
        mov PauseCpu,FALSE  
    .endif
    
    ret

LoadRom endp

;
; Intialize the Emulator
;
InitEmulator proc uses eax
    LOCAL hMenu,hSubMenu:HMENU
    
    ;get the right menu items so we can check/uncheck them
    invoke GetMenu,g_MainhWnd
    mov hMenu,eax
    invoke GetSubMenu,hMenu,1
    mov hSubMenu,eax
    invoke GetSubMenu,hSubMenu,1
    mov hSubMenu,eax
    
    ;Flag Sound/Pause
    mov SoundOn,TRUE
    mov PauseCpu,FALSE
    
    ; Check/Uncheck Menu Items
    invoke CheckMenuItem,hMenu,ID_PAUSE,MF_UNCHECKED
    invoke CheckMenuItem,hSubMenu,ID_SOUND_ON,MF_CHECKED
    invoke CheckMenuItem,hSubMenu,ID_SOUND_OFF,MF_UNCHECKED
    
    ret
InitEmulator endp

;
; Single Step Mode For Debugging 
;
CPU_STEP proc
    
    ; Execute 1 Opcode per debug process click.
    invoke CPU
    
    ;Check/Update the Timers
    .if (DelayTimer>0)
       dec DelayTimer
    .endif
        
    .if (SoundTimer>0)
      .if(SoundOn==TRUE && SoundTimer==1)
        invoke Beep,150,40 ; Buzz Beep Sound at speakers
      .endif
            
      dec SoundTimer
    .endif
    
    ret
CPU_STEP endp

;
; Intialize the CPU
;
Init_CPU proc uses ecx esi edi
    LOCAL i:DWORD
    
    mov CodeIndex,0200h ; Execution of ROM Starts at 200h
    ;Reset Timers,I Register, SP
    mov DelayTimer,0
    mov SoundTimer,0
    mov RegI,0
    mov StackPointer,0
    
    ;Copy Font to Memory
    mov ecx,FONT_SIZE/SIZEOF DWORD
    mov edi,offset Memory
    mov esi,offset Font
    cld
    rep movsd ; Copy
    
    ;Reset the content of buffers to 0
    invoke RtlZeroMemory,addr Registers,MAX_REGS
    invoke RtlZeroMemory,addr Screen,SCREEN_SIZE
    Invoke RtlZeroMemory,addr Stack,MAX_STACK
    
    ret

Init_CPU endp

;
; Emulation Main Loop
;
Emulate proc uses eax ecx    
    ; Emulation Loop
    .if (EmulatorRunning==TRUE && PauseCpu==FALSE)
    
        ;Emulation Speed        
        mov ecx,ClockSpeed
        imul ecx,100000
        jecxz _Emulate
@@:
        dec ecx
        jnz @B       
_Emulate:        
        invoke CPU ; Run the CPU
        
        .if (DelayTimer>0)
            dec DelayTimer
        .endif
        
        .if (SoundTimer>0)
            .if(SoundOn==TRUE && SoundTimer==1)
                invoke Beep,150,40 ; Buzz Beep Sound at speakers
            .endif
            
            dec SoundTimer
        .endif
        
    .endif
    
    ret

Emulate endp

;
; CPU CORE
;
CPU proc uses eax edx ebx
    LOCAL op:WORD
    LOCAL wOpcode:WORD
    LOCAL Opcode:BYTE
    
    ;initialize
    xor eax,eax
    xor ebx,ebx
    xor edx,edx
    mov wOpcode,0
    mov Opcode,0
    mov op,0
    
    mov ebx,CodeIndex
    mov ax,WORD PTR Memory[ebx]
    shl ax,8
    mov dx,WORD PTR Memory[ebx+1]
    and dx,00FFh
    or ax,dx
    mov op,ax
    mov wOpcode,ax ; Opcode to Execute
    and ax,0F000h
    shr ax,12
    mov Opcode,al ; Opcode Format (0-Fh)
    
    ; --==========--
    ; Opcode Table
    ; --==========--
    .if (Opcode==0)
        mov ax,wOpcode
        and ax,00FFh
        
        .if (al==0E0h)
           invoke Chip8_cls
        .elseif (al==0EEh)
           invoke Chip8_rts          
        .elseif (al==0FBh)
            add CodeIndex,2 ; SCHIP8
            
        .elseif (al==0FCh)
            add CodeIndex,2 ; SCHIP8
            
        .elseif (al==0FEh)
            add CodeIndex,2 ; SCHIP8
            
        .elseif (al==0FFh)
            add CodeIndex,2 ; SCHIP8
        .else
            add CodeIndex,2 ; SCHIP8
        .endif
        
    .elseif (Opcode==1)
        invoke Chip8_jmp,wOpcode
        
    .elseif (Opcode==2)
        invoke Chip8_jsr,wOpcode
        
    .elseif (Opcode==3)
        invoke Chip8_Skip_Reg_eq_Constant,wOpcode
        
    .elseif (Opcode==4)
        invoke Chip8_Skip_Reg_not_eq_Constant,wOpcode
        
    .elseif (Opcode==5)
        invoke Chip8_Skip_Reg_r_eq_Reg_y,wOpcode
        
    .elseif (Opcode==6)
        invoke Chip8_Mov_Reg_r_Constant,wOpcode
        
    .elseif (Opcode==7)
        invoke Chip8_Add_Reg_r_Constant,wOpcode
        
    .elseif (Opcode==8)
        mov ax,wOpcode
        and ax,000Fh
        .if (al==0)
            invoke Chip8_Mov_Reg_y_to_Reg_r,wOpcode
            
        .elseif (al==1)
            invoke Chip8_Or_Reg_y_to_Reg_r, wOpcode
            
        .elseif (al==2)
            invoke Chip8_And_Reg_y_to_Reg_r,wOpcode
            
        .elseif (al==3)
            invoke Chip8_Xor_Reg_y_to_Reg_r,wOpcode
            
        .elseif (al==4)
            invoke Chip8_Add_Reg_y_to_Reg_r,wOpcode
            
        .elseif (al==5)
            invoke Chip8_Sub_Reg_y_to_Reg_r,wOpcode
            
        .elseif (al==6)
            invoke Chip8_Shift_Left_Reg_r,wOpcode
            
        .elseif (al==7)
            invoke Chip8_Reg_r_rsb_Reg_y,wOpcode
        .elseif (al==0Eh)   
            invoke Chip8_Shift_Right_Reg_r,wOpcode
        .endif
    .elseif (Opcode==9)
         invoke Chip8_Skip_Reg_r_not_eq_Reg_y,wOpcode
         
    .elseif (Opcode==0Ah)
         invoke Chip8_Load_RegI_with_Constant,wOpcode
         
    .elseif (Opcode==0Bh)
        invoke Chip8_Jmp_To_Address_with_Reg0,wOpcode
        
    .elseif (Opcode==0Ch)
        invoke Chip8_Random_Number_less_or_eq_Constant,wOpcode
        
    .elseif (Opcode==0Dh)
         mov ax,wOpcode
         and ax,000Fh
         .if (al==0)
            invoke Chip8_Put_16it_Pixel_On_Screen,wOpcode
         .else
            invoke Chip8_Put_8Bit_Pixel_On_Screen,wOpcode
         .endif
    .elseif (Opcode==0Eh)
        mov ax,wOpcode
        and ax,00FFh 
        .if (al==09Eh)
            invoke Chip8_Skip_if_key_pressed,wOpcode
        .elseif (al==0A1h) 
            invoke Chip8_Skip_if_key_not_pressed,wOpcode  
        .endif
    .elseif (Opcode==0Fh) 
        mov ax,wOpcode
        and ax,00FFh 
        .if (al==7)
            invoke Chip8_Get_delay_timer,wOpcode
        .elseif (al==0Ah)
            invoke Chip8_Scan_Keys,wOpcode
        .elseif (al==15h)
            invoke Chip8_Set_delay_timer,wOpcode
        .elseif (al==18h)
            invoke Chip8_Set_sound_timer,wOpcode
        .elseif (al==1Eh)
            invoke Chip8_Add_Reg_r_to_Reg_i,wOpcode
        .elseif (al==29h)
            invoke Chip8_Set_i_to_hex_character,wOpcode
        .elseif (al==30h)
            invoke Chip8_Set_i_to_hex_character_super,wOpcode
        .elseif (al==33h)
            invoke Chip8_Store_bcd_at_ram,wOpcode
        .elseif (al==55h)
            invoke Chip8_Set_Regs_at_Memory,wOpcode
        .elseif (al==65h)    
            invoke Chip8_Load_Regs_from_Memory,wOpcode    
        .endif            
    .endif
    
    ret

CPU endp

;
; Chip-8 Disassembler
;
DisasmChip8 proc uses eax edi ecx edx wOpcode:WORD, StrPTR:DWORD
    LOCAL Opcode:BYTE
    LOCAL WOpcode:WORD
    
    mov edi, StrPTR
    mXOR eax
    push wOpcode ;
    pop WOpcode  ; Copy Vars
    
    and WOpcode,0F000h
    shr WOpcode,12
    mov ax,WOpcode
    mov Opcode,al ; Opcode Format (0-Fh)
    push wOpcode  
    pop WOpcode   ; Copy Vars
    
    mXOR eax
    mXOR ecx
    mXOR edx
    ; --==========--
    ; Opcode Table
    ; --==========--
    .if (Opcode==0)
        mov ax,wOpcode
        and ax,00FFh
        
        .if (al==0E0h)
           invoke lstrcpy,edi,offset Str_CLS
        .elseif (al==0EEh)
           invoke lstrcpy,edi,offset Str_RTS          
        .elseif (al==0FBh)
            invoke lstrcpy,edi,offset Str_SCRIGHT
            
        .elseif (al==0FCh)
            invoke lstrcpy,edi,offset Str_SCLEFT
            
        .elseif (al==0FEh)
            invoke lstrcpy,edi,offset Str_LOW
            
        .elseif (al==0FFh)
            invoke lstrcpy,edi,offset Str_HIGH
        .else
            and WOpcode,0Fh
            mov ax,WOpcode
            invoke wsprintf,edi,offset Str_SCDOWN,eax
        .endif
        
    .elseif (Opcode==1)
        and WOpcode,0FFFh
        mov ax,WOpcode
        invoke wsprintf,edi,offset Str_JMP,eax
        
    .elseif (Opcode==2)
        and WOpcode,0FFFh
        mov ax,WOpcode
        invoke wsprintf,edi,offset Str_JMP,eax
        
    .elseif (Opcode==3)
        and WOpcode,0F00h
        shr WOpcode,8
        and wOpcode,00FFh
        mov ax,WOpcode
        mov cx,wOpcode 
        invoke wsprintf,edi,offset Str_SKEQ,eax,ecx
        
    .elseif (Opcode==4)
        and WOpcode,0F00h
        shr WOpcode,8
        and wOpcode,00FFh
        mov ax,WOpcode
        mov cx,wOpcode 
        invoke wsprintf,edi,offset Str_SKNE,eax,ecx
        
    .elseif (Opcode==5)
        and WOpcode,0F00h
        shr WOpcode,8
        and wOpcode,00F0h
        shr wOpcode,4
        mov ax,WOpcode
        mov cx,wOpcode         
        invoke wsprintf,edi,offset Str_SKEQ2,eax,ecx
        
    .elseif (Opcode==6)
        and WOpcode,0F00h
        shr WOpcode,8
        and wOpcode,00FFh
        mov ax,WOpcode
        mov cx,wOpcode         
        invoke wsprintf,edi,offset Str_MOV,eax,ecx
        
    .elseif (Opcode==7)
        and WOpcode,0F00h
        shr WOpcode,8
        and wOpcode,00FFh
        mov ax,WOpcode
        mov cx,wOpcode         
        invoke wsprintf,edi,offset Str_ADD,eax,ecx
        
    .elseif (Opcode==8)
        mov ax,wOpcode
        and ax,000Fh
        .if (al==0)
         and WOpcode,0F00h
         shr WOpcode,8
         and wOpcode,00F0h
         shr wOpcode,4
         mov ax,WOpcode
         mov cx,wOpcode          
         invoke wsprintf,edi,offset Str_MOV2,eax,ecx
            
        .elseif (al==1)
         and WOpcode,0F00h
         shr WOpcode,8
         and wOpcode,00F0h
         shr wOpcode,4
         mov ax,WOpcode
         mov cx,wOpcode          
         invoke wsprintf,edi,offset Str_OR,eax,ecx
            
        .elseif (al==2)
         and WOpcode,0F00h
         shr WOpcode,8
         and wOpcode,00F0h
         shr wOpcode,4
         mov ax,WOpcode
         mov cx,wOpcode          
         invoke wsprintf,edi,offset Str_AND,eax,ecx
            
        .elseif (al==3)
         and WOpcode,0F00h
         shr WOpcode,8
         and wOpcode,00F0h
         shr wOpcode,4
         mov ax,WOpcode
         mov cx,wOpcode          
         invoke wsprintf,edi,offset Str_XOR,eax,ecx
            
        .elseif (al==4)
         and WOpcode,0F00h
         shr WOpcode,8
         and wOpcode,00F0h
         shr wOpcode,4
         mov ax,WOpcode
         mov cx,wOpcode          
         invoke wsprintf,edi,offset Str_ADD2,eax,ecx
            
        .elseif (al==5)
         and WOpcode,0F00h
         shr WOpcode,8
         and wOpcode,00F0h
         shr wOpcode,4
         mov ax,WOpcode
         mov cx,wOpcode          
         invoke wsprintf,edi,offset Str_SUB,eax,ecx
            
        .elseif (al==6)
         and WOpcode,0F00h
         shr WOpcode,8
         mov ax,WOpcode
         invoke wsprintf,edi,offset Str_SHR,eax
            
        .elseif (al==7)
         and WOpcode,0F00h
         shr WOpcode,8
         and wOpcode,00F0h
         shr wOpcode,4
         mov ax,WOpcode
         mov cx,wOpcode          
         invoke wsprintf,edi,offset Str_RSB,eax,ecx
         
        .elseif (al==0Eh)   
         and WOpcode,0F00h
         shr WOpcode,8
         mov ax,WOpcode
         invoke wsprintf,edi,offset Str_SHL,eax
         
        .endif
        
    .elseif (Opcode==9)
         and WOpcode,0F00h
         shr WOpcode,8
         and wOpcode,00F0h
         shr wOpcode,4
         mov ax,WOpcode
         mov cx,wOpcode          
         invoke wsprintf,edi,offset Str_SKNE2,eax,ecx
         
    .elseif (Opcode==0Ah)
         and WOpcode,0FFFh
         mov ax,WOpcode
         invoke wsprintf,edi,offset Str_MVI,eax
         
    .elseif (Opcode==0Bh)
         and WOpcode,0FFFh
         mov ax,WOpcode
         invoke wsprintf,edi,offset Str_JMI,eax
        
    .elseif (Opcode==0Ch)
         and WOpcode,0F00h
         shr WOpcode,8
         and wOpcode,0FFFh
         mov ax,WOpcode
         mov cx,wOpcode          
         invoke wsprintf,edi,offset Str_RAND,eax,ecx
        
    .elseif (Opcode==0Dh)
         mov ax,wOpcode
         and ax,000Fh
         .if (al==0)
          and WOpcode,0F00h
          shr WOpcode,8
          and wOpcode,00F0h
          shr wOpcode,4
          mov ax,WOpcode
          mov cx,wOpcode           
          invoke wsprintf,edi,offset Str_XSPRITE,eax,ecx
         
         .else
           mXOR eax
           mov ax,WOpcode
           and WOpcode,0F00h
           shr WOpcode,8
           and wOpcode,00F0h
           shr wOpcode,4
           and eax,0Fh
           mov dx,WOpcode
           mov cx,wOpcode
           invoke wsprintf,edi,offset Str_SPRITE,edx,ecx,eax
         .endif
         
    .elseif (Opcode==0Eh)
        mov ax,wOpcode
        and ax,00FFh 
        .if (al==09Eh)
          and WOpcode,0F00h
          shr WOpcode,8
          mov ax,WOpcode
          invoke wsprintf,edi,offset Str_SKPR,eax
          
        .elseif (al==0A1h) 
          and WOpcode,0F00h
          shr WOpcode,8
          mov ax,WOpcode
          invoke wsprintf,edi,offset Str_SKUP,eax
          
        .endif
        
    .elseif (Opcode==0Fh) 
        mov ax,wOpcode
        and ax,00FFh 
        .if (al==7)
          and WOpcode,0F00h
          shr WOpcode,8
          mov ax,WOpcode          
          invoke wsprintf,edi,offset Str_GDELAY,eax
          
        .elseif (al==0Ah)
          and WOpcode,0F00h
          shr WOpcode,8
          mov ax,WOpcode          
          invoke wsprintf,edi,offset Str_KEY,eax
          
        .elseif (al==15h)
          and WOpcode,0F00h
          shr WOpcode,8
          mov ax,WOpcode          
          invoke wsprintf,edi,offset Str_SDELAY,eax
          
        .elseif (al==18h)
          and WOpcode,0F00h
          shr WOpcode,8
          mov ax,WOpcode          
          invoke wsprintf,edi,offset Str_SSOUND,eax
          
        .elseif (al==1Eh)
          and WOpcode,0F00h
          shr WOpcode,8
          mov ax,WOpcode          
          invoke wsprintf,edi,offset Str_ADI,eax
          
        .elseif (al==29h)
          and WOpcode,0F00h
          shr WOpcode,8
          mov ax,WOpcode          
          invoke wsprintf,edi,offset Str_FONT,eax
          
        .elseif (al==30h)
          and WOpcode,0F00h
          shr WOpcode,8
          mov ax,WOpcode          
          invoke wsprintf,edi,offset Str_XFONT,eax
          
        .elseif (al==33h)
          and WOpcode,0F00h
          shr WOpcode,8
          mov ax,WOpcode          
          invoke wsprintf,edi,offset Str_BCD,eax
          
        .elseif (al==55h)
          and WOpcode,0F00h
          shr WOpcode,8
          mov ax,WOpcode          
          invoke wsprintf,edi,offset Str_STR,eax
          
        .elseif (al==65h)    
          and WOpcode,0F00h
          shr WOpcode,8
          mov ax,WOpcode          
          invoke wsprintf,edi,offset Str_LDR,eax
            
        .endif            
    .endif

    ret
DisasmChip8 endp


;
; Clear the Screen
;
Chip8_cls proc
    invoke RtlZeroMemory,addr Screen,SCREEN_SIZE
    add CodeIndex,2    
    ret
Chip8_cls endp

;
; Return to Subroutine
;
Chip8_rts proc uses eax esi   
    xor esi,esi
    xor eax,eax
    dec StackPointer
    mov esi,StackPointer
    imul esi,SIZEOF DWORD
    mov eax, dword ptr Stack[esi]
    mov CodeIndex,eax
    add CodeIndex,2
    ret
Chip8_rts endp

;
; Jmp to Address
;
Chip8_jmp proc uses eax wOpcode:WORD
    xor eax,eax
    mov ax,wOpcode
    and ax,0FFFh
    mov CodeIndex,eax
    ret
Chip8_jmp endp

;
; Jump to Subroutine
;
Chip8_jsr proc uses eax esi wOpcode:WORD
    mov eax,CodeIndex
    mov esi,StackPointer  ; Index
    imul esi,SIZEOF DWORD ; Calculate Index*sizeof dword in Array of Dwords
    mov dword ptr Stack[esi], eax ; Store Current Address
    add StackPointer,1 ; New Index
    xor eax,eax
    mov ax,wOpcode
    and ax,0FFFh
    mov CodeIndex,eax  ; Jump to New Address 
    ret
Chip8_jsr endp    

;
; Skip Next Instruction if Reg[R] == XX
;
Chip8_Skip_Reg_eq_Constant proc uses esi ebx wOpcode:WORD   
    GET_REG wOpcode    
    xor ebx,ebx
    mov bx,wOpcode
    and bx,00FFh
    .if (byte ptr Registers[esi] == bl)
        add CodeIndex,4
    .else
        add CodeIndex,2
    .endif
    
    ret
Chip8_Skip_Reg_eq_Constant endp

;
; Skip Next Instruction if Reg[R] != XX
;
Chip8_Skip_Reg_not_eq_Constant proc uses esi ebx wOpcode:WORD 
    GET_REG wOpcode    
    xor ebx,ebx
    mov bx,wOpcode
    and bx,00FFh
    .if (byte ptr Registers[esi] != bl)
        add CodeIndex,4
    .else
        add CodeIndex,2
    .endif        
    ret
Chip8_Skip_Reg_not_eq_Constant endp

;
; Skip Next Instruction if Reg[R] == Reg[Y]
;
Chip8_Skip_Reg_r_eq_Reg_y proc uses esi edi eax wOpcode:WORD    
    GET_REG wOpcode
    GET_REG2 wOpcode
    xor eax,eax
    mov al,byte ptr Registers[edi]
    .if byte ptr Registers[esi]==al
       add CodeIndex,4 
    .else
       add CodeIndex,2
    .endif
    ret
Chip8_Skip_Reg_r_eq_Reg_y endp

;
; Move Constant to Register, Reg[R] = XX
;
Chip8_Mov_Reg_r_Constant proc uses esi eax wOpcode:WORD
    GET_REG wOpcode
    xor eax,eax
    mov ax,wOpcode
    and ax,00FFh
    mov byte ptr Registers[esi],al
    add CodeIndex,2
    ret
Chip8_Mov_Reg_r_Constant endp

;
; Add Constant to Register, Reg[R] += XX
;
Chip8_Add_Reg_r_Constant proc uses esi eax wOpcode:WORD
    GET_REG wOpcode
    xor eax,eax
    mov ax,wOpcode
    and ax,00FFh
    add byte ptr Registers[esi],al
    add CodeIndex,2
    ret
Chip8_Add_Reg_r_Constant endp

;
; Move Registers, Reg[R] = Reg[Y]
;
Chip8_Mov_Reg_y_to_Reg_r proc uses esi edi eax wOpcode:WORD
    GET_REG wOpcode
    GET_REG2 wOpcode
    xor eax,eax
    mov al,byte ptr Registers[edi]
    mov byte ptr Registers[esi],al
    add CodeIndex,2
    ret
Chip8_Mov_Reg_y_to_Reg_r endp    

;
; OR Registers, Reg[R] = Reg[R] OR Reg[Y]
;
Chip8_Or_Reg_y_to_Reg_r proc uses esi edi eax wOpcode:WORD
    GET_REG wOpcode
    GET_REG2 wOpcode
    xor eax,eax
    mov al,byte ptr Registers[edi]
    or byte ptr Registers[esi],al
    add CodeIndex,2
    ret
Chip8_Or_Reg_y_to_Reg_r endp 

;
; AND Registers, Reg[R] = Reg[R] AND Reg[Y]
;
Chip8_And_Reg_y_to_Reg_r proc uses esi edi eax wOpcode:WORD
    GET_REG wOpcode
    GET_REG2 wOpcode
    xor eax,eax
    mov al,byte ptr Registers[edi]
    and byte ptr Registers[esi],al
    add CodeIndex,2
    ret
Chip8_And_Reg_y_to_Reg_r endp 

;
; XOR Registers, Reg[R] = Reg[R] XOR Reg[Y]
;
Chip8_Xor_Reg_y_to_Reg_r proc uses esi edi eax wOpcode:WORD
    GET_REG wOpcode
    GET_REG2 wOpcode
    xor eax,eax
    mov al,byte ptr Registers[edi]
    xor byte ptr Registers[esi],al
    add CodeIndex,2
    ret
Chip8_Xor_Reg_y_to_Reg_r endp 

;
; ADD Registers, Reg[R] = Reg[R] + Reg[Y]
;
Chip8_Add_Reg_y_to_Reg_r proc uses eax edi esi ebx wOpcode:WORD
    GET_REG wOpcode
    GET_REG2 wOpcode
    xor eax,eax
    xor ebx,ebx
    mov al,0FFh
    sub al,byte ptr Registers[esi]
    mov bl,byte ptr Registers[edi]
    
    .if (bl>al)
       mov byte ptr Registers[0Fh],1 
    .else
       mov byte ptr Registers[0Fh],0
    .endif
    
    add byte ptr Registers[esi],bl
    add CodeIndex,2
    ret
Chip8_Add_Reg_y_to_Reg_r endp

;
; SUB Registers, Reg[R] = Reg[R] - Reg[Y]
;
Chip8_Sub_Reg_y_to_Reg_r proc uses eax ebx esi edi wOpcode:WORD
    GET_REG wOpcode
    GET_REG2 wOpcode
    
    mov al,byte ptr Registers[esi]
    mov bl,byte ptr Registers[edi]
    
    .if (al < byte ptr Registers[edi])
        mov byte ptr Registers[0Fh],0
    .else
        mov byte ptr Registers[0Fh],1      
    .endif
    
    sub byte ptr Registers[esi],bl
    add CodeIndex,2
    ret
Chip8_Sub_Reg_y_to_Reg_r endp

;
; SHIFT RIGHT Register, Reg[R] = Reg[R] SHR 1
;
Chip8_Shift_Right_Reg_r proc uses eax ebx esi edi wOpcode:WORD
    GET_REG wOpcode
    xor eax,eax
    xor ebx,ebx
    mov al,byte ptr Registers[esi]  
    mov bl,al  
    and al,01h
    mov byte ptr Registers[0Fh],al
    shr bl,1
    mov byte ptr Registers[esi],bl
    add CodeIndex,2
    ret
Chip8_Shift_Right_Reg_r endp    

;
; SUB Registers with CARRY, Reg[R] = Reg[R] - Reg[Y]
;
Chip8_Reg_r_rsb_Reg_y proc uses eax ebx esi edi wOpcode:WORD
    GET_REG wOpcode
    GET_REG2 wOpcode
    xor eax,eax
    xor ebx,ebx
    mov al, byte ptr Registers[edi]
    mov bl, byte ptr Registers[esi]
    
    .if (al < bl)
       mov byte ptr Registers[0Fh],1
    .else
       mov byte ptr Registers[0Fh],0 
    .endif
    
    sub al,bl
    mov byte ptr Registers[si],al
    add CodeIndex,2
    ret
Chip8_Reg_r_rsb_Reg_y endp    

;
; SHIFT LEFT Register, Reg[R] = Reg[R] SHL 1
;
Chip8_Shift_Left_Reg_r proc uses eax ebx esi edi wOpcode:WORD
    GET_REG wOpcode    
    mXOR eax
    mXOR ebx
    mov al,byte ptr Registers[esi]
    mov bl,al
    and al,80h
    shr al,7
    mov byte ptr Registers[0Fh],al
    shr bl,1
    mov byte ptr Registers[esi],bl
    add CodeIndex,2    
    ret
Chip8_Shift_Left_Reg_r endp

;
; Skip Next Instruction if Reg[R] != Reg[Y]
;
Chip8_Skip_Reg_r_not_eq_Reg_y proc uses eax ebx edi esi wOpcode:WORD
    GET_REG wOpcode
    GET_REG2 wOpcode
    mXOR eax
    mXOR ebx
    mov al,byte ptr Registers[esi]
    .if al!=byte ptr Registers [edi]
        add CodeIndex,4
    .else
        add CodeIndex,2
    .endif    
    ret
Chip8_Skip_Reg_r_not_eq_Reg_y endp

;
; I = XX
;
Chip8_Load_RegI_with_Constant proc uses eax wOpcode:WORD
    mXOR eax
    mov ax,wOpcode
    and ax,0FFFh
    mov RegI,eax
    add CodeIndex,2    
    ret
Chip8_Load_RegI_with_Constant endp

;
; Jmp to: V[0] + XXX
;
Chip8_Jmp_To_Address_with_Reg0 proc uses eax ebx wOpcode:WORD
    mXOR eax
    mXOR ebx
    mov ax,wOpcode
    and ax,0FFFh
    mov bl,byte ptr Registers[0] ; Register V[0]
    add ax,bx
    mov CodeIndex,eax
    ret
Chip8_Jmp_To_Address_with_Reg0 endp

;
; Create Random Number Between Min - Max
;
RandomNumGen PROC uses ebx edx min:DWORD, max:DWORD
    LOCAL RandSeed:DWORD
    
    invoke GetTickCount
    mov RandSeed,eax    
    mov     ebx, max
    sub     ebx, min
    inc     ebx
    xor     edx, edx
    mov     eax, RandSeed
    div     ebx
    add     RandSeed, eax
    add     edx, min
    mov     eax, edx
    ret

RandomNumGen ENDP

;
; Reg[R] = Rnd()%XX
;
Chip8_Random_Number_less_or_eq_Constant proc uses eax wOpcode:WORD
    GET_REG wOpcode
    mXOR eax
    mov ax,wOpcode
    and ax,00FFh
    invoke RandomNumGen,0,eax
    mov byte ptr Registers[esi],al
    add CodeIndex,2
    ret
Chip8_Random_Number_less_or_eq_Constant endp

;
; Reg[R] = Delay Timer
;
Chip8_Get_delay_timer proc uses eax wOpcode:WORD
    GET_REG wOpcode
    mXOR eax
    mov al,DelayTimer
    mov byte ptr Registers[esi],al
    add CodeIndex,2    
    ret
Chip8_Get_delay_timer endp

;
; Delay Timer = Reg[R]
;
Chip8_Set_delay_timer proc uses eax wOpcode:WORD
    GET_REG wOpcode
    mXOR eax
    mov al,byte ptr Registers[esi]
    mov DelayTimer,al
    add CodeIndex,2
    ret
Chip8_Set_delay_timer endp

;
; Scan for Pressed Keys
;
Chip8_Scan_Keys proc uses eax esi edx wOpcode:WORD    
    invoke ScanKeys
    GET_REG wOpcode
    mov byte ptr Registers[esi],al
    add CodeIndex,2
    ret
Chip8_Scan_Keys endp

;
; Use DirectInput for checking Key Presses
;
ScanKeys proc
    mXOR eax
    DIDINVOKE  Acquire,m_keyboard
    DIDINVOKE GetDeviceState,m_keyboard,SIZEOF keys,addr keys
    KeyDown DIK_NUMPAD0,12,TRUE
    KeyDown DIK_NUMPAD1,8,TRUE
    KeyDown DIK_NUMPAD2,9,TRUE
    KeyDown DIK_NUMPAD3,10,TRUE
    KeyDown DIK_NUMPAD4,4,TRUE
    KeyDown DIK_NUMPAD5,5,TRUE
    KeyDown DIK_NUMPAD6,6,TRUE
    KeyDown DIK_NUMPAD7,0,TRUE
    KeyDown DIK_NUMPAD8,1,TRUE
    KeyDown DIK_NUMPAD9,2,TRUE
    KeyDown DIK_NUMPADSLASH,3,TRUE
    KeyDown DIK_NUMPADSTAR,7,TRUE
    KeyDown DIK_NUMPADMINUS,11,TRUE
    KeyDown DIK_DELETE,13,TRUE
    KeyDown DIK_RETURN,14,TRUE
    KeyDown DIK_NUMPADPLUS,15,TRUE       
    ret
ScanKeys endp

;
; Skip next instruction if Key has been pressed
;
Chip8_Skip_if_key_pressed proc uses eax ebx wOpcode:WORD
    GET_REG wOpcode
    xor eax,eax
    xor ebx,ebx
    mov al,byte ptr Registers[esi]
    mov bl, byte ptr Keys[eax]
    DIDINVOKE  Acquire,m_keyboard
    DIDINVOKE GetDeviceState,m_keyboard,SIZEOF keys,addr [keys]
    KeyDown ebx,FALSE,FALSE
    .if eax!=NULL
        add CodeIndex,4
    .else
        add CodeIndex,2
    .endif
    ret
Chip8_Skip_if_key_pressed endp

;
; Skip next instruction if Key has not been pressed
;
Chip8_Skip_if_key_not_pressed proc uses eax ebx wOpcode:WORD
    GET_REG wOpcode
    xor eax,eax
    xor ebx,ebx
    mov al,byte ptr Registers[esi]
    mov bl, byte ptr Keys[eax]
    DIDINVOKE  Acquire,m_keyboard
    DIDINVOKE GetDeviceState,m_keyboard,SIZEOF keys,addr [keys]
    KeyDown ebx,FALSE,FALSE
    .if eax==NULL
        add CodeIndex,4
    .else
        add CodeIndex,2
    .endif
    ret
Chip8_Skip_if_key_not_pressed endp

;
; Sound Timer = Reg[R] 
;
Chip8_Set_sound_timer proc uses eax wOpcode:WORD
    GET_REG wOpcode
    xor eax,eax
    mov al,byte ptr Registers[esi]
    mov SoundTimer,al
    add CodeIndex,2
    ret
Chip8_Set_sound_timer endp

;
; I = Reg[R] + I
;
Chip8_Add_Reg_r_to_Reg_i proc wOpcode:WORD
    GET_REG wOpcode
    xor eax,eax
    mov al,byte ptr Registers[esi]
    add RegI,eax
    add CodeIndex,2    
    ret
Chip8_Add_Reg_r_to_Reg_i endp

;
; I = Reg [R] * 5
;
Chip8_Set_i_to_hex_character proc uses eax wOpcode:WORD
    GET_REG wOpcode
    xor eax,eax
    mov al,byte ptr Registers[esi]    
    imul eax,5
    mov RegI,eax
    add CodeIndex,2    
    ret
Chip8_Set_i_to_hex_character endp

;
; I = Reg [R] * 10
;
Chip8_Set_i_to_hex_character_super proc uses eax wOpcode:WORD
    GET_REG wOpcode
    xor eax,eax
    mov al,byte ptr Registers[esi]
    imul eax,10    
    mov RegI,eax
    add CodeIndex,2    
    ret
Chip8_Set_i_to_hex_character_super endp

;
; Store BCD Representation of Reg[R] at memory location pointed by I
;
; Memory [I]   = Reg[R]/100
; Memory [I+1] = (Reg[R]/10) MOD 10
; Memory [I+2] = (Reg[R] MOD 100) MOD 10
;
Chip8_Store_bcd_at_ram proc uses eax ecx esi edi wOpcode:WORD
    GET_REG wOpcode
    xor eax,eax
    mov al,byte ptr Registers[esi]
    mov bl,al
    mov edi,RegI
    ;(x/100)
    mov ecx,100
    div cx
    mov byte ptr Memory[edi],al
    inc edi ; next byte
    ;(x/10)%10;
    xor eax,eax
    mov al,bl
    mov ecx,10
    div cl
    div cl
    mov byte ptr Memory[edi],ah
    inc edi
    ;(x%100)%10
    xor eax,eax
    mov al,bl
    mov ecx,100
    div ecx
    mov eax,edx
    mov ecx,10
    div cl
    mov byte ptr Memory[edi],ah
    
    add CodeIndex,2
    ret
Chip8_Store_bcd_at_ram endp

;
; Save all Registers at Memory
;
Chip8_Set_Regs_at_Memory proc uses ecx eax edi edx wOpcode:WORD
    GET_REG wOpcode
    mov ecx,esi        
    xor eax,eax
    mov edi,RegI    
    .repeat
        xor edx,edx
        mov byte ptr dl, Registers [eax]
        mov byte ptr Memory[edi+eax],dl
        inc eax
        jecxz No_Load ; for(i=0;i<=Reg[R];i++)
        dec ecx
    .until (ecx == 0)
No_Load:    
    add CodeIndex,2
    ret
Chip8_Set_Regs_at_Memory endp

;
; Load The Registers from memory
;
Chip8_Load_Regs_from_Memory proc uses ecx eax edi edx wOpcode:WORD
    GET_REG wOpcode
    mov ecx,esi      
    mXOR eax
    mov edi,RegI    
    .repeat
        xor edx,edx
        mov byte ptr dl, Memory [edi+eax]
        mov byte ptr Registers[eax],dl
        inc eax
        jecxz No_Load ; for(i=0;i<=Reg[R];i++)
        dec ecx
    .until ecx==0
No_Load:    
    add CodeIndex,2
    ret
Chip8_Load_Regs_from_Memory endp

;
; Loop The Screen and put pixels (Black/White) 8Bit
;
Chip8_Put_8Bit_Pixel_On_Screen proc uses eax edi esi ecx wOpcode:WORD
    LOCAL height,x,y,row,line:DWORD
    LOCAL Pixel:BYTE
    
    GET_REG wOpcode
    GET_REG2 wOpcode
    
    mov y,0
    mov x,0
    mov height,0
    mXOR eax
    mov ax,wOpcode
    and eax,000Fh
    mov height,eax
    mXOR eax
    mov al,byte ptr Registers[esi]
    mov x,eax
    mXOR eax
    mov al,byte ptr Registers[edi]
    mov y,eax     
    mov row,0         
        
loop_row:
    mXOR eax
    mov eax,row  ; Current Row
    mov edi,RegI ; Pixel Index
    mov al,byte ptr Memory [edi+eax] ; Get Pixel Byte Data (XXXX XXXX)
    mov Pixel,al ; Save Pixel Data
    mov line,0   ; Init Loop counter
loop_line:
    mov eax,080h  ; check all bits, start from bit7 (1000 0000)
    mov ecx,line  ; Bit Position
    shr eax,cl    ; Scan each Bit in the pixel data and see if its on Bit=1
    test Pixel,al ; test if bit is On or Off
    jz Pixel_OFF  ; Bit is Off.
Pixel_ON:
    ; Calculate Position on screen: line+x+( (y+row)*64 )
    mXOR eax
    mov eax,y
    add eax,row
    imul eax,64
    add eax,x
    add eax,line
    .if (eax<=SCREEN_SIZE) ; making sure we write into the boundry of the Screen buffer
        ; Check Collision
        .if (byte ptr Screen[eax] == 1)  
            mov byte ptr Registers[0Fh],1 ; Set Carry on Collision
        .endif 
        xor byte ptr Screen[eax],1 ; Toggle Pixel On / Off
    .endif
Pixel_OFF:         
    add line,1  
    cmp line,8
    jne loop_line
    
    add row,1
    mov eax,row
    cmp eax,height
    jne loop_row
    
    add CodeIndex,2
    invoke DrawScene ;Draw the Screen Buffer into The DX Surface
    ret
Chip8_Put_8Bit_Pixel_On_Screen endp

;
; Loop The Screen and put pixels (Black/White) 16Bit
;
Chip8_Put_16it_Pixel_On_Screen proc uses eax edi esi ecx wOpcode:WORD
    LOCAL x,y,row,line:DWORD
    LOCAL Pixel:BYTE
    
    GET_REG wOpcode
    GET_REG2 wOpcode
    
    mXOR eax
    mov al,byte ptr Registers[esi]
    mov x,eax
    mXOR eax
    mov al,byte ptr Registers[edi]
    mov y,eax     
    mov row,0 
        
loop_row:
    mov eax,row
    mov edi,RegI
    mov al,byte ptr Memory [edi+eax] ; al == pixel
    mov Pixel,al
    mov line,0
loop_line:
    mov eax,080h
    mov ecx,line
    shr eax,cl
    test Pixel,al
    jz Pixel_OFF
Pixel_ON:
    xor eax,eax
    mov eax,y
    add eax,row
    imul eax,64
    add eax,x
    add eax,line
    ; Check Collision
    .if (byte ptr Screen[eax] == 1)
        mov byte ptr Registers[0Fh],1 ; Carry on Collision
    .endif
    xor byte ptr Screen[eax],1       
Pixel_OFF:
    add line,1  
    cmp line,16
    jne loop_line
    
    add row,1
    mov eax,row
    cmp eax,16
    jne loop_row
    
    add CodeIndex,2
    invoke DrawScene ;Draw the Screen Buffer onto DX Surface
    ret
Chip8_Put_16it_Pixel_On_Screen endp