;
;PROTOTYPES
;
InitEmulator PROTO
LoadRom      PROTO :HWND
Init_CPU     PROTO
CPU_STEP     PROTO
Emulate      PROTO
CPU          PROTO
DisasmChip8  PROTO :WORD,:DWORD
RandomNumGen PROTO :DWORD,:DWORD
ScanKeys     PROTO

;CPU Functions
Chip8_cls                               PROTO
Chip8_rts                               PROTO
Chip8_jmp                               PROTO :WORD
Chip8_jsr                               PROTO :WORD
Chip8_Skip_Reg_eq_Constant              PROTO :WORD
Chip8_Skip_Reg_not_eq_Constant          PROTO :WORD
Chip8_Skip_Reg_r_eq_Reg_y               PROTO :WORD
Chip8_Mov_Reg_r_Constant                PROTO :WORD
Chip8_Add_Reg_r_Constant                PROTO :WORD
Chip8_Mov_Reg_y_to_Reg_r                PROTO :WORD
Chip8_Or_Reg_y_to_Reg_r                 PROTO :WORD
Chip8_And_Reg_y_to_Reg_r                PROTO :WORD
Chip8_Xor_Reg_y_to_Reg_r                PROTO :WORD
Chip8_Add_Reg_y_to_Reg_r                PROTO :WORD 
Chip8_Sub_Reg_y_to_Reg_r                PROTO :WORD
Chip8_Shift_Right_Reg_r                 PROTO :WORD
Chip8_Reg_r_rsb_Reg_y                   PROTO :WORD
Chip8_Shift_Left_Reg_r                  PROTO :WORD
Chip8_Skip_Reg_r_not_eq_Reg_y           PROTO :WORD
Chip8_Load_RegI_with_Constant           PROTO :WORD
Chip8_Jmp_To_Address_with_Reg0          PROTO :WORD
Chip8_Random_Number_less_or_eq_Constant PROTO :WORD
Chip8_Get_delay_timer                   PROTO :WORD
Chip8_Set_delay_timer                   PROTO :WORD
Chip8_Skip_if_key_pressed               PROTO :WORD
Chip8_Skip_if_key_not_pressed           PROTO :WORD
Chip8_Set_sound_timer                   PROTO :WORD
Chip8_Add_Reg_r_to_Reg_i                PROTO :WORD
Chip8_Set_i_to_hex_character            PROTO :WORD
Chip8_Set_i_to_hex_character_super      PROTO :WORD
Chip8_Store_bcd_at_ram                  PROTO :WORD
Chip8_Scan_Keys                         PROTO :WORD
Chip8_Set_Regs_at_Memory                PROTO :WORD
Chip8_Load_Regs_from_Memory             PROTO :WORD
Chip8_Put_16it_Pixel_On_Screen          PROTO :WORD
Chip8_Put_8Bit_Pixel_On_Screen          PROTO :WORD

;
;DEIFNES
;
MEMORY_SIZE = 0FFFh
SCREEN_SIZE = 2049
MAX_REGS    = 16
MAX_STACK   = MAX_REGS
FONT_SIZE   = 80

; EMULATOR KEYS
KEY_0        = 052h
KEY_1        = 04Fh
KEY_2        = 050h
KEY_3        = 051h
KEY_4        = 04Bh
KEY_5        = 04Ch
KEY_6        = 04Dh
KEY_7        = 047h
KEY_8        = 048h
KEY_9        = 049h
KEY_SLASH    = 0B5h
KEY_ASTERISK = 037h
KEY_MINUS    = 04Ah
KEY_DEL      = 0D3h
KEY_ENTER    = 01Ch
KEY_PLUS     = 04Eh

;
;MACROS
;
GET_REG macro Opcode
    ; (Opcode&0F00)>>8
    xor esi,esi
    mov si,Opcode
    and si,0F00h
    shr si,8 
endm

GET_REG2 macro Opcode
    ; (Opcode&00F0)>>4
    xor edi,edi
    mov di,Opcode
    and di,00F0h
    shr di,4 
endm

mXOR macro REG
   IRP R,<EAX,ECX,EDX,ESI,EDI,EBX>
      IFIDNI <R>, <REG>
        xor REG,REG
        EXITM
      ENDIF
   ENDM
endm

;
;DATA
;
.data
ClockSpeed  DWORD  0
FileFilter db "All Files (*.*)",0,"*.*",0,0
szFileName db MAX_PATH dup(0)
;
; Disasm Strings
;
Str_CLS     db "cls",0
Str_RTS     db "rts",0
Str_SCRIGHT db "scright",0
Str_SCLEFT  db "scleft",0
Str_LOW     db "low",0
Str_HIGH    db "high",0
Str_SCDOWN  db "scdown %d",0
Str_JMP     db "jmp %03X",0
Str_JSR     db "jsr %03X",0
Str_SKEQ    db "skeq v%d,%02X",0
Str_SKNE    db "skne v%d,%02X",0
Str_SKEQ2   db "skeq v%d,v%d",0
Str_MOV     db "mov v%d,%02X",0
Str_ADD     db "add v%d,%02X",0
Str_MOV2    db "mov v%d,v%d",0
Str_OR      db "or v%d,v%d",0
Str_AND     db "and v%d,v%d",0
Str_XOR     db "xor v%d,v%d",0
Str_ADD2    db "add v%d,v%d",0
Str_SUB     db "sub v%d,v%d",0
Str_SHR     db "shr v%d",0
Str_RSB     db "rsb v%d,v%d",0
Str_SHL     db "shl v%d",0
Str_SKNE2   db "skne v%d,v%d",0
Str_MVI     db "mvi %03X",0
Str_JMI     db "jmi %03X",0
Str_RAND    db "rand v%d,%03X",0
Str_XSPRITE db "xsprite %d,%d",0
Str_SPRITE  db "sprite %d,%d,%d",0
Str_SKPR    db "skpr %d",0
Str_SKUP    db "skup %d",0
Str_GDELAY  db "gdelay v%d",0
Str_KEY     db "key v%d",0
Str_SDELAY  db "sdelay v%d",0
Str_SSOUND  db "ssound v%d",0
Str_ADI     db "adi v%d",0
Str_FONT    db "font v%d",0
Str_XFONT   db "xfont v%d",0
Str_BCD     db "bcd v%d",0
Str_STR     db "str v0-v%d",0
Str_LDR     db "ldr v0-v%d",0


COMMENT *

DECODING/ENCODDING CHIP-8 8 BIT FONTS
 
Example:
         Letter: 'F' == 0F0h, 080h, 0F0h, 080h, 080h
         
         Bit 1 = Black
         Bit 0 = White 
         
         11110000 ; 0F0h
         10000000 ; 080h
         11110000 ; 0F0h
         10000000 ; 080h
         10000000 ; 080h
         
         Letter 'A' == 0F0h, 090h, 0F0h, 090h, 090h
         
         11110000 ; 0F0h
         10010000 ; 090h
         11110000 ; 0F0h
         10010000 ; 090h
         10010000 ; 090h
         
/COMMENT*

Font       db 0F0h, 090h, 090h, 090h, 0F0h ; 0
           db 020h, 060h, 020h, 020h, 070h ; 1
           db 0F0h, 010h, 0F0h, 080h, 0F0h ; 2
           db 0F0h, 010h, 0F0h, 010h, 0F0h ; 3
           db 090h, 090h, 0F0h, 010h, 010h ; 4
           db 0F0h, 080h, 0F0h, 010h, 0F0h ; 5
           db 0F0h, 080h, 0F0h, 090h, 0F0h ; 6
           db 0F0h, 010h, 020h, 040h, 040h ; 7
           db 0F0h, 090h, 0F0h, 090h, 0F0h ; 8
           db 0F0h, 090h, 0F0h, 010h, 0F0h ; 9
           db 0F0h, 090h, 0F0h, 090h, 090h ; A
           db 0E0h, 090h, 0E0h, 090h, 0E0h ; B
           db 0F0h, 080h, 080h, 080h, 0F0h ; C
           db 0E0h, 090h, 090h, 090h, 0E0h ; D
           db 0F0h, 080h, 0F0h, 080h, 0F0h ; E
           db 0F0h, 080h, 0F0h, 080h, 080h ; F

; Keys Setup as originaly used by the CHIP8           
Keys   db KEY_7, KEY_8,  KEY_9,     KEY_SLASH
	   db KEY_4, KEY_5,  KEY_6,     KEY_ASTERISK
	   db KEY_1, KEY_2,  KEY_3,     KEY_MINUS
	   db KEY_0, KEY_DEL,KEY_ENTER, KEY_PLUS
                
.data?
Memory         db MEMORY_SIZE dup(?)
Screen         db SCREEN_SIZE dup(?)
Registers      db MAX_REGS    dup(?)
Stack          dd MAX_STACK   dup(?)
CodeIndex      DWORD    ?
StackPointer   DWORD    ?
RegI           DWORD    ?
DisasmThreadId DWORD    ?
hDisasmThread  HANDLE   ?
DelayTimer     db       ?
SoundTimer     db       ?        